/*
    qtZenOscillo : Software for using DSO-2020 usb scope under Linux
    Copyright (C) 2015 Olivier Poitevin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact : code@zener.fr

*/

#include "scopeview.h"
#include "ui_scopeview.h"
#include <QPainter>
#include <qmath.h>

ScopeView::ScopeView(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ScopeView)
{
    ui->setupUi(this);
    qRegisterMetaType<QVector<double> >("QVector<double>");
    connect(&dThread,SIGNAL(dataReady(QVector<double>,QVector<double>,int)),this,SLOT(getData(QVector<double>,QVector<double>,int)));
    ch1Activated=false;
    ch2Activated=false;
    fastMode=false;
    trigChan=1;
    trigValue=0.0;
    nbSamplesToDisplay=0;
    trigFallingEdge=false;
    trigFiltered=false;
    ch1Offset=0.0;
    ch2Offset=0.0;
    lastMouseX=0;
    viewOffset=0;
    speedM=0;
    speedU=0;
}

ScopeView::~ScopeView()
{
    // stop thread
    ToggleRun(false);
    delete ui;
}

void ScopeView::getData(QVector<double> chan1,QVector<double> chan2,int nbToDisplay)
{
    chan1Points=chan1;
    chan2Points=chan2;
    nbSamplesToDisplay=nbToDisplay;
    update();
}

void ScopeView::drawGrid(QPainter &p,int left,int top,int right,int bottom)
{
    int gridWidth=right-left;
    int gridHeight=bottom-top;

    p.setPen(Qt::gray);
    QPen dashPen(Qt::gray);
    dashPen.setStyle(Qt::DashLine);
    p.setPen(dashPen);

    for(int i=0;i<=10;i++)
    {
        if(i!=5 && i!=0 && i!=10)
            p.setPen(dashPen);
        else
            p.setPen(Qt::gray);
        p.drawLine(left,top+(gridHeight*i)/10,right,top+(gridHeight*i)/10);
        p.drawLine(left+(gridWidth*i)/10,top,left+(gridWidth*i)/10,bottom);
    }

}

void ScopeView::drawChannel(QPainter &p,int chanNum,enum Qt::GlobalColor pen,int sampleOffset,int left,int top,int width,int height,QVector<double>&Dat)
{
    static const char *unitLut[]={"ns","us","ms","s"};
    int xLast,yLast,nbToDisplay;
    int xOffset=left;
    int yOffset=top+height/2;


    p.setPen(pen);

    nbToDisplay=nbSamplesToDisplay;
    if((nbToDisplay+sampleOffset)>Dat.size())
        nbToDisplay=Dat.size()-sampleOffset;

    double xRatio=(double)width/nbToDisplay;
    double yRatio=(double)height/2.0;

    int viewSampleOffset=(int)((double)viewOffset/xRatio);

    if((viewSampleOffset+sampleOffset)<0)
    {
        viewSampleOffset=-sampleOffset;
        viewOffset=(int)((double)viewSampleOffset*xRatio);
    }
    else if((nbToDisplay+sampleOffset+viewSampleOffset)>Dat.size())
    {
        viewSampleOffset=Dat.size()-nbToDisplay-sampleOffset;
        viewOffset=(int)((double)viewSampleOffset*xRatio);
    }

    if(chanNum==1)
    {
        yOffset-=yRatio*ch1Offset;
    }
    else
    {
        yOffset-=yRatio*ch2Offset;
    }

    // display trigger value
    if(!fastMode && trigChan==chanNum)
    {
        QPen dashPen(pen);
        dashPen.setStyle(Qt::DashDotDotLine);
        p.setPen(dashPen);
        p.drawLine(left,yOffset-yRatio*trigValue,left+width,yOffset-yRatio*trigValue);
        p.setPen(pen);
    }


    // display channel
    for(int i=0;i<nbToDisplay;i++)
    {
        int x,y;
        x=xOffset+(int)(xRatio*(double)i);
        y=yOffset-(int)(Dat.at(i+sampleOffset+viewSampleOffset)*yRatio);
        if(i)
        {
            p.drawLine(xLast,yLast,x,y);
        }
        xLast=x;
        yLast=y;
    }

    if(viewOffset)
    {
        double timeOffset=viewOffset/(double)width*(double)speedM*10.0;
        p.setPen(Qt::white);
        p.drawText(100,0,200,100,0,"Ofst:" + QString::number(timeOffset,'f',2) + unitLut[speedU]);
    }
}

void ScopeView::paintEvent(QPaintEvent*)
{
    int xL,xR,yT,yB,width,height;

    QPainter p(this);
    QRect geometry=this->geometry();
    xL=0;
    yT=0;
    xR=geometry.width();
    yB=geometry.height();

    // paint back black
    p.setBrush(Qt::black);
    p.setPen(Qt::black);
    p.drawRect(0,0,xR,yB);

    // margins :
    xL+=20;
    xR-=20;
    yT+=20;
    yB-=20;

    width=xR-xL;
    height=yB-yT;

    drawGrid(p,xL,yT,xR,yB);

    // find start sample if trigger
    int startSample=0;
    if(!fastMode)
    {
        int nbSamples;
        double lastVal,filteredVal;
        QVector<double>*V;
        if(trigChan==1)
            V=&chan1Points;
        else
            V=&chan2Points;
        if(V->size())
        {
            double filterCoefA,filterCoefB;
            double trigValueLow,trigValueHigh;
            nbSamples=V->size()-nbSamplesToDisplay;
            if(trigFiltered)
            {
                filterCoefA=1.0-(50.0/(double)nbSamplesToDisplay);
                if(filterCoefA>1.0)
                    filterCoefA=1.0;
                else if(filterCoefA<0.1)
                    filterCoefA=0.1;
            }
            else
            {
                filterCoefA=0.0;
            }
            filterCoefB=1.0-filterCoefA;
            // hysteresis
            trigValueLow=trigValue-0.001;
            trigValueHigh=trigValue+0.001;

            lastVal=V->at(nbSamplesToDisplay/2);
            filteredVal=lastVal;
            for(int i=nbSamplesToDisplay/2;i<nbSamples;i++)
            {
                double curVal=V->at(i);
                filteredVal=filteredVal*filterCoefA+filterCoefB*curVal;
                if((!trigFallingEdge && lastVal<trigValueLow && filteredVal>=trigValueHigh)||
                        (trigFallingEdge && lastVal>trigValueHigh && filteredVal<=trigValueLow))
                {
                    startSample=i;
                    break;
                }
                if(filteredVal<trigValueLow || filteredVal>trigValueHigh)
                    lastVal=filteredVal;
            }
            startSample-=nbSamplesToDisplay/2;
            if(startSample<0)
                startSample=0;
        }
    }
    if(ch1Activated && chan1Points.size())
    {
        drawChannel(p,1,Qt::yellow,startSample,xL,yT,width,height,chan1Points);
    }
    if(ch2Activated && chan2Points.size())
    {
        drawChannel(p,2,Qt::green,startSample,xL,yT,width,height,chan2Points);
    }
}


void ScopeView::mousePressEvent(QMouseEvent * event)
{
     if(event->buttons()&Qt::LeftButton)
     {
         // cliking
        lastMouseX=viewOffset+event->pos().x();
     }
}

void ScopeView::mouseMoveEvent(QMouseEvent * event)
{
     if(event->buttons()&Qt::LeftButton)
     {
        viewOffset=lastMouseX-event->pos().x();
        update();
     }
}


bool ScopeView::ToggleRun(bool state)
{
    if(dThread.isRunning() && !state)
    {
        dThread.doStop();
        return false;
    }
    if(!dThread.isRunning() && state)
    {
        if(!dThread.initialize())
            return false;
        dThread.start();
        return true;
    }
    return dThread.isRunning();
}

void ScopeView::setSpeed(int mantissa,int unit)
{
    speedM=mantissa;
    speedU=unit;
    viewOffset=0;
    chan1Points.clear();
    chan2Points.clear();
    dThread.setSpeed(mantissa,unit);
    update();
}

void ScopeView::setCh1Activated(bool value)
{
    ch1Activated=value;
    dThread.setCh1Activated(value);
    update();
}

void ScopeView::setCh2Activated(bool value)
{
    ch2Activated=value;
    dThread.setCh2Activated(value);
    update();
}

void ScopeView::setCh1Range(int mV)
{
    chan1Points.clear();
    dThread.setCh1Range(mV);
    update();
}


void ScopeView::setCh2Range(int mV)
{
    chan2Points.clear();
    dThread.setCh2Range(mV);
    update();
}

void ScopeView::setFastMode(bool value)
{
    fastMode=value;
    dThread.setFastMode(value);
}

void ScopeView::setTrigChan(int chan)
{
    trigChan=chan;
    update();
}

void ScopeView::setTrigValue(double val)
{
    trigValue=val;
    update();
}

void ScopeView::setEdgeType(bool isFalling)
{
    trigFallingEdge=isFalling;
    update();
}

void ScopeView::setTrigFilter(bool state)
{
    trigFiltered=state;
    update();
}

void ScopeView::setCh1Offset(double val)
{
    ch1Offset=val;
    update();
}

void ScopeView::setCh2Offset(double val)
{
    ch2Offset=val;
    update();
}
